function handle = drawInvolute(P,N,PA,x,y,phi)
% handle = drawInvolute(P,N,PA,x,y,phi)
% 
% This MATLAB function draws a spur gear with involute teeth.
% P = diametral pitch
% N = number of teeth
% PA = pressure angle in radians
% x, y are linear offset of center
% phi is rotational offset in radians

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  
%  Copyright 2009, 2010 Matt Moses   
%  mmoses152 at earthlink dot net
% 
%  This file is part of Multigear. 
%
%  Multigear is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  Multigear is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with Multigear.  If not, see <http://www.gnu.org/licenses/>.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The following websites were helpful in providing definitions of 
% terminology and formulas for gear geometry:
%
% The Involute Curve, Drafting a Gear in CAD and Applications
% by Nick Carter 
% http://www.cartertools.com/involute.html
%
% Introduction to Mechanisms
% Chapter 7 Gears
% by Yi Zhang, with Susan Finger and Stephannie Behrens 
% http://www.cs.cmu.edu/~rapidproto/mechanisms/chpt7.html
%
% Dynamic Mechanisms Tutorial 
% http://www.ul.ie/~nolk/gears.htm#GEARS

D = N/P; % pitch diameter
DB = D * cos(PA); % base circle diameter
a = 1/P; % addendum (also known as "module")
d = 1.25/P; % dedendum (this is set by a standard)
DO = D + 2*a; % outside diameter
DR = D - 2*d; % root diameter

% angular thickness of gear where profile intersects pitch circle
theta_mod = pi/N;

% angular coordinate at intersection of profile and pitch circle
theta_p = ccinv(D/2, DB/2);

% The involute profile is approximated by connected line segments
% nseg is the number of line segments in the involute profile 
% of one side of the tooth
nseg = 10;

% list of radius values at which we will calculate angular coordinates
r_pro1 = linspace(DB/2,DO/2,nseg);

% now clip r_pro1 in case the base circle is smaller than the root circle
r_temp = (DR/2)*(r_pro1 < DR/2) + r_pro1 .* (r_pro1 >= DR/2);
r_pro1 = r_temp;

% angular coordinates corresponding to radius values
t_pro1 = ccinv(r_pro1, DB/2);

% Rtooth and Ttooth are polar coordinates for one tooth of the gear.
% 'R' as in Radius, 'T' as in Theta
Rtooth = [DR/2, r_pro1, r_pro1(end:-1:1),DR/2];
Ttooth = [t_pro1(1), t_pro1, theta_mod + 2*theta_p - t_pro1(end:-1:1), ...
    theta_mod + 2*theta_p - t_pro1(1)];

% This is the angular thickness of the 'body' of one tooth,
% used to get phase alignment right.
angThick = Ttooth(end) - Ttooth(1);

% Initialize vectors for the whole gear
bigT = [];
bigR = [];

% This incrementally adds coordinates for the rest of the teeth to the
% vectors bigT and bigR
for i = 1:N
    bigT = [bigT, Ttooth + 2*(i-1)*theta_mod];
    bigR = [bigR, Rtooth];
end

% convert polar coordinates to XY coordinates, and incorporate phase
% rotation and shifting of the center
bigX = [bigR bigR(1)] .* cos([bigT bigT(1)] - angThick/2 + phi) + x;
bigY = [bigR bigR(1)] .* sin([bigT bigT(1)] - angThick/2 + phi) + y;

% This draws the gear
handle = line(bigX, bigY);